<?php

use Zen\Payment\CartData;
use Zen\Payment\Paywall;
use Zen\Payment\Util;

require 'payment-core/ZenHelper.php';
require 'payment-core/vendor/autoload.php';

class zen
{

	const VERSION = '1.0.1';

	var $code, $title, $description, $enabled;

	function zen()
	{
		global $order;

		$this->code = 'zen';
		$this->title = MODULE_PAYMENT_ZEN_TEXT_TITLE;
		$this->description = MODULE_PAYMENT_ZEN_TEXT_DESCRIPTION;

		$this->sort_order = MODULE_PAYMENT_ZEN_SORT_ORDER;

		$this->enabled = true;

		if((int) MODULE_PAYMENT_ZEN_ORDER_STATUS_ID > 0) {
			$this->order_status = MODULE_PAYMENT_ZEN_ORDER_STATUS_ID;
		}

		if(is_object($order)) {
			$this->update_status();
		}
	}

	function update_status()
	{
		$this->enabled = true;

		return;
	}

	function javascript_validation()
	{
		return false;
	}

	function selection()
	{
		return [
			'id'     => $this->code,
			'module' => $this->title,
		];
	}

	function pre_confirmation_check()
	{
		return false;
	}

	function confirmation()
	{
		return false;
	}

	function process_button()
	{
		return false;
	}

	function before_process()
	{
		return false;
	}

	function getItemsArray($order)
	{
		$items = [];
		foreach($order->products as $orderItem) {
			$items[] = [
				'name'   => $orderItem['name'],
				'amount' => $orderItem['final_price'] * $orderItem['qty'],
				'qty'    => (integer) $orderItem['qty'],
				'price'  => $orderItem['final_price'],
			];
		}

		$orderDetails = $order->info;
		$items[] = [
			'name'   => $orderDetails['shipping_method'],
			'amount' => $orderDetails['shipping_cost'],
			'qty'    => 1,
			'price'  => $orderDetails['shipping_cost'],
		];

		if($orderDetails['tax'] > 0) {
			$items[] = [
				'name'   => MODULE_PAYMENT_ZEN_TEXT_TAX,
				'amount' => $orderDetails['tax'],
				'qty'    => 1,
				'price'  => $orderDetails['tax'],
			];
		}

		$discountAmount = $_SESSION['order_summary']['credits_applied'];
		if($discountAmount > 0) {
			$items[] = [
				'name'   => MODULE_PAYMENT_ZEN_TEXT_ROUND,
				'amount' => -$discountAmount,
				'qty'    => 1,
				'price'  => -$discountAmount,
			];
		}

		return $items;
	}

	function after_process()
	{
		global $insert_id, $order, $cart;

		$paywall = new Paywall(
			Util::ENVIRONMENT_PRODUCTION,
			MODULE_PAYMENT_ZEN_PAYWALL_SECRET,
			MODULE_PAYMENT_ZEN_TERMINAL_UUID
		);

		$cartData = new CartData();
		$currencyValue = (float) $order->info['currency_value'];

		foreach($this->getItemsArray($order) as $key => $value) {

			$cartData->addItem(
				$value['name'],
				Util::convertAmountToFractional($value['price'] * $currencyValue),
				$value['qty'],
				Util::convertAmountToFractional($value['amount'] * $currencyValue)
			);
		}

		$total = Util::convertAmountToFractional((float) $order->info['total'] * $currencyValue);
		$cartData->setAmount($total);

		$form = $paywall->buildOrderForm(
			Paywall::prepareData(
				$total,
				$order->info['currency'],
				$insert_id,
				$order->customer['firstname'],
				$order->customer['lastname'],
				$order->customer['email_address'],
				tep_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'),
				tep_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $insert_id, 'SSL'),
				tep_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $insert_id, 'SSL'),
				tep_href_link('zenipn.php', '', 'SSL'),
				$cartData->getCartData(MODULE_PAYMENT_ZEN_TEXT_ROUND)
			)
		);

		if(!$form) {
			return false;
		}

		$cart->reset(true);

		echo MODULE_PAYMENT_ZEN_TEXT_REDIRECT
			. '<br/>'
			. $form
			. '<script type="text/javascript">'
			. 'document.getElementById(\'submit-payment-form\').style.display = "none";'
			. 'document.getElementById(\'submit-payment-form\').click();'
			. '</script>';
		die();
	}

	function get_error()
	{
		return false;
	}

	function output_error()
	{
		return false;
	}

	function check()
	{

		if(!isset($this->_check)) {
			$this->_check = tep_db_num_rows(tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_ZEN_STATUS'"));
		}

		return $this->_check;
	}

	function install()
	{

		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable ZEN module', 'MODULE_PAYMENT_ZEN_STATUS', 'True', 'Enable easy payments via blik, pbl, etc.', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Terminal UUID', 'MODULE_PAYMENT_ZEN_TERMINAL_UUID', '', '', '6', '3', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Paywall Secret', 'MODULE_PAYMENT_ZEN_PAYWALL_SECRET', '', '', '6', '4', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('IPN Secret', 'MODULE_PAYMENT_ZEN_IPN_SECRET', '', '', '6', '5', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Unpaid Order Status', 'MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID', '" . intval(DEFAULT_ORDERS_STATUS_ID) . "', 'Automatically set the status of unpaid orders to this value.', '6', '7', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Paid Order Status', 'MODULE_PAYMENT_ZEN_PAID_STATUS_ID', '2', 'Automatically set the status of paid orders to this value.', '6', '8', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Display order', 'MODULE_PAYMENT_ZEN_SORT_ORDER', '0', 'Display order. The lowest are displayed first', '6', '10', now())");
		tep_db_query('CREATE TABLE IF NOT EXISTS ' . ZenHelper::ZEN_CONFIRMED_TRANSACTION_TABLE_NAME . ' (`id` INT NOT NULL AUTO_INCREMENT, `order_id` INT NOT NULL, `transaction_id` VARCHAR(50) NOT NULL, PRIMARY KEY (`id`))');
	}

	function remove()
	{
		tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
	}

	function keys()
	{
		return [
			'MODULE_PAYMENT_ZEN_STATUS',
			'MODULE_PAYMENT_ZEN_TERMINAL_UUID',
			'MODULE_PAYMENT_ZEN_PAYWALL_SECRET',
			'MODULE_PAYMENT_ZEN_IPN_SECRET',
			'MODULE_PAYMENT_ZEN_UNPAID_STATUS_ID',
			'MODULE_PAYMENT_ZEN_PAID_STATUS_ID',
			'MODULE_PAYMENT_ZEN_SORT_ORDER',
		];
	}
}
